% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Generate Table 7, Table 8, Figure 2 and Figure 3]

% method = 'ate'    ATE with adjusted standard error
%          'sfe'    QTE with stratified fixed effects 
%          'qte'    QTE with naive standard error
%          'qadj'   QTE with adjusted standard error
%          'qsfe'   QTE with stratified fixed effects
%          'ipw'    ATE IPW with adjusted standard error
%          'qipw'   QTE IPW with adjusted standard error
%          'bnaive' ATE&QTE with weighted bootstrap standard error
%          'cab'    ATE&QTE with covariate-adaptive bootstrap standard error

clear;
clc;

chong_pure = importdata('Chong_pure.csv');
data = chong_pure.data; 
clear chong_pure

S  = data(:,1);
A  = data(:,5);
Y_grade = data(:,4);
Anemic  = data(:,6);
A  = A.*(A==1);
Anemic1 = (Anemic==1);
Anemic0 = (Anemic==0);
Y  = Y_grade;
Anemic = Anemic0;
Y(Anemic,:) = [];
A(Anemic,:) = [];
S(Anemic,:) = [];


ns1 = unique(S);
DS1_sup = 0;
for i=1:ns1
    S_index = (S==i);
    A1_temp = A(S_index);
    DS1_sup = max(DS1_sup,abs(mean(A1_temp-0.5)));  
end
result_q0    = chong_fn_pure(Y, S, A, 'q');
result_qipw0 = chong_fn_pure(Y, S, A, 'qipw');
result_bnaive0 = chong_fn_pure(Y, S, A, 'bnaive');
result_ate0    = chong_fn_pure(Y, S, A, 'ate');
result_ipw0    = chong_fn_pure(Y, S, A, 'ipw');

S  = data(:,1);
A  = data(:,5);
A  = A.*(A==1);
Y  = Y_grade;
Anemic = Anemic1;
Y(Anemic,:) = [];
A(Anemic,:) = [];
S(Anemic,:) = [];

ns0 = unique(S);
DS0_sup = 0;
for i=1:ns0
    S_index = (S==i);
    A0_temp = A(S_index);
    DS0_sup = max(DS0_sup,abs(mean(A0_temp-0.5)));  
end

result_q1    = chong_fn_pure(Y, S, A, 'q');
result_qipw1 = chong_fn_pure(Y, S, A, 'qipw');
result_bnaive1 = chong_fn_pure(Y, S, A, 'bnaive');
result_ate1    = chong_fn_pure(Y, S, A, 'ate');
result_ipw1    = chong_fn_pure(Y, S, A, 'ipw');

S  = data(:,1);
A  = data(:,5);
Y  = Y_grade;

result_ate    = chong_fn_pure(Y, S, A, 'ate');
result_sfe    = chong_fn_pure(Y, S, A, 'sfe');
result_ipw    = chong_fn_pure(Y, S, A, 'ipw');
result_qipw = chong_fn_pure(Y, S, A, 'qipw');
result_bnaive = chong_fn_pure(Y, S, A, 'bnaive');
result_cab = chong_fn_pure(Y, S, A, 'cab');
result_qsfe = chong_fn_pure(Y, S, A, 'qsfe');
result_q = chong_fn_pure(Y, S, A, 'q');

taulist = 0.1:0.05:0.9;
qx_ax = [taulist, fliplr(taulist)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clf;

figure(1);
subplot(2,2,1);
q1x    = [result_q.beta', fliplr(result_q.beta')];
q1y_lb = result_q.beta' - 1.96*result_bnaive.qstd(:,1)';
q1y_ub = result_q.beta' + 1.96*result_bnaive.qstd(:,1)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
    set(gca,'FontSize',14);
    title('s/W');
    hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off

subplot(2,2,2);

q1x    = [result_q.beta', fliplr(result_q.beta')];
q1y_lb = result_q.beta' - 1.96*result_cab.qstd(:,1)';
q1y_ub = result_q.beta' + 1.96*result_cab.qstd(:,1)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
    set(gca,'FontSize',14);
    title('s/CA');
    hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off


subplot(2,2,3);

q1x    = [result_qipw.beta, fliplr(result_qipw.beta)];
q1y_lb = result_qipw.beta - 1.96*result_bnaive.qstd(:,3)';
q1y_ub = result_qipw.beta + 1.96*result_bnaive.qstd(:,3)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
    set(gca,'FontSize',14);
    title('ipw/W');
    hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off


subplot(2,2,4);

q1x    = [result_qipw.beta, fliplr(result_qipw.beta)];
q1y_lb = result_qipw.beta - 1.96*result_cab.qstd(:,3)';
q1y_ub = result_qipw.beta + 1.96*result_cab.qstd(:,3)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
    set(gca,'FontSize',14);
    title('ipw/CA');
    hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off

saveas(gcf,'figure2.fig')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(3);
subplot(2,2,1);
q1x    = [result_q0.beta', fliplr(result_q0.beta')];
q1y_lb = result_q0.beta' - 1.96*result_bnaive0.qstd(:,1)';
q1y_ub = result_q0.beta' + 1.96*result_bnaive0.qstd(:,1)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
set(gca,'FontSize',15);
title('s/W, Anemic');
hold on
line([0.1,0.9],[0,0],'linestyle','--')


hold off
subplot(2,2,2);
q1x    = [result_q1.beta', fliplr(result_q1.beta')];
q1y_lb = result_q1.beta' - 1.96*result_bnaive1.qstd(:,1)';
q1y_ub = result_q1.beta' + 1.96*result_bnaive1.qstd(:,1)';
q1y  = [q1y_lb, fliplr(q1y_ub)];
hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
set(gca,'FontSize',15);
title('s/W, Nonanemic');
hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off

subplot(2,2,3);
q1x    = [result_qipw0.beta, fliplr(result_qipw0.beta)];
q1y_lb = result_qipw0.beta - 1.96*result_bnaive0.qstd(:,2)';
q1y_ub = result_qipw0.beta + 1.96*result_bnaive0.qstd(:,2)';
q1y  = [q1y_lb, fliplr(q1y_ub)];

hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
set(gca,'FontSize',15);
title('ipw/W, Anemic');
hold on
line([0.1,0.9],[0,0],'linestyle','--')


hold off
subplot(2,2,4);
q1x    = [result_qipw1.beta, fliplr(result_qipw1.beta)];
q1y_lb = result_qipw1.beta - 1.96*result_bnaive1.qstd(:,2)';
q1y_ub = result_qipw1.beta + 1.96*result_bnaive1.qstd(:,2)';
q1y  = [q1y_lb, fliplr(q1y_ub)];
hold on
plot(qx_ax, q1x, 'blue', 'LineWidth', 1.2)
fill(qx_ax, q1y , 1,....
        'facecolor','blue', ...
        'edgecolor','none', ...
        'facealpha', 0.2);
set(gca,'FontSize',15);
title('ipw/W, Nonanemic');
hold on
line([0.1,0.9],[0,0],'linestyle','--')
hold off

saveas(gcf,'figure3.fig')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FID = fopen('table7.tex', 'w');
fprintf(FID, 'ATE & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f)  \\\\ ',result_ate.beta, result_sfe.std(1), result_ate.beta, result_bnaive.std(1), result_ipw.beta, result_bnaive.std(3),result_ate.beta, result_bnaive.std(1), result_ipw.beta, result_bnaive.std(3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,25  &  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f)  \\\\ ',result_q.beta(4), result_bnaive.qstd(4,1), result_q.beta(4), result_cab.qstd(4,1), result_qipw.beta(4), result_bnaive.qstd(4,3), result_qipw.beta(4), result_cab.qstd(4,3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,50  &  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) \\\\ ', result_q.beta(9), result_bnaive.qstd(9,1), result_q.beta(9), result_cab.qstd(9,1), result_qipw.beta(9), result_bnaive.qstd(9,3), result_qipw.beta(9), result_cab.qstd(9,3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,75 &  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) \\\\ ', result_q.beta(14), result_bnaive.qstd(14,1), result_q.beta(14), result_cab.qstd(14,1), result_qipw.beta(14), result_bnaive.qstd(14,3), result_qipw.beta(14), result_cab.qstd(14,3));
fprintf(FID, '\n');
fclose(FID);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FID = fopen('table8.tex', 'w');
fprintf(FID, 'ATE  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f)  \\\\ ',  result_ate0.beta, result_bnaive0.std(1), result_ipw0.beta, result_bnaive0.std(3),result_ate1.beta, result_bnaive1.std(1), result_ipw1.beta, result_bnaive1.std(3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,25  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f)  \\\\ ',  result_q0.beta(4), result_bnaive0.qstd(4,1), result_qipw0.beta(4), result_bnaive0.qstd(4,3),result_q1.beta(4), result_bnaive1.qstd(4,1), result_qipw1.beta(4), result_bnaive1.qstd(4,3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,50  & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) \\\\ ',  result_q0.beta(9), result_bnaive0.qstd(9,1), result_qipw0.beta(9), result_bnaive0.qstd(9,3),result_q1.beta(9), result_bnaive1.qstd(9,1), result_qipw1.beta(9), result_bnaive1.qstd(9,3));
fprintf(FID, '\n');
fprintf(FID, 'QTE,75 & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) & %8.2f(%8.2f) \\\\ ',  result_q0.beta(14), result_bnaive0.qstd(14,1),result_qipw0.beta(14), result_bnaive0.qstd(14,3),result_q1.beta(14), result_bnaive1.qstd(14,1), result_qipw1.beta(14), result_bnaive1.qstd(14,3));
fprintf(FID, '\n');
fclose(FID);


