% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Generate Treatment Assignment]


% idx2: index of the assignment rule.
%       idx2=1 for SRS; idx2=2 for WEI; idx2=3 for BCD; idx2=4 for SBR.
% S   : covariates
% pi  : pi, targeted assignment probability. 



function A = AR(idx2,S,pi)
n = length(S);g = max(S);A = zeros(n,1);

if idx2 == 1
    A = (rand(n,1)<pi);
elseif idx2 == 3
    A(1) = (rand(1,1)<0.5);
    for k = 2:n
        D = sum((A(1:k-1)-0.5).*(S(1:k-1) == S(k)));
        p = 0.5*(D==0) + 0.75*(D<0) + 0.25*(D>0);
        A(k) = (rand(1,1)<p);
    end
        
elseif idx2 == 2
    A(1) = (rand(1,1)<0.5);
    for k = 2:n
        D = 2*sum((A(1:k-1)-0.5).*(S(1:k-1) == S(k)));
        p = (1-D/(sum(S(1:k-1) == S(k))))/2;
        A(k) = (rand(1,1)<p);
    end
elseif idx2 == 4
    A = zeros(n,1);
    for s = 1:g
        count1 = (S==s);
        idx = zeros(n,1);
        for i = 1:n
            idx(i,1) = sum(count1(1:i));
        end
        count2 = [(1:n)',count1.*idx];
        ns = sum(count1);
        ms = floor(pi*ns);
        idxa = randsample(ns,ms,false)';
        idxd = zeros(n,1);
        for i = 1:ms
            idxd = idxd+(count2(:,2)==idxa(i));
        end
        A(idxd==1,1)=1;
        
    end
    
end
