% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Data Generating Function]

% idx1: index of the dgp. 
%       idx1=1 for method 1; idx1=3 for method 2.
% idx2: index of the assignment rule.
%       idx2=1 for SRS; idx2=2 for WEI; idx2=3 for BCD; idx2=4 for SBR.
% n: sample size
% g: number of intervals. 
% pi: pi, targeted assignment probability. 
% gamma: coefficient of Z. 
% sigma: standard error of the error term for Y(1)

function [Y,S,A] = func_DGP(idx1,idx2,n,g,mu0,mu1,pi,gamma,sigma)
if idx1 == 1
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);
A = AR(idx2,S,pi);
eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y = mu0+(mu1-mu0)*A+gamma*Z + eta;

elseif idx1 == 2
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);
A = AR(idx2,S,pi);
eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y = mu0+(mu1-mu0)*A+gamma*Z.*A - gamma*(1-A).*(log(Z+3).*(Z<=0.5)) + eta;

elseif idx1 == 3
Z = 4*(rand(n,1)-0.5);
m0 = gamma*Z.^2.*(Z<=1).*(Z>=-1)+ gamma*(2-Z.^2).*(1-(Z<=1).*(Z>=-1))/4;
m1 = -m0;
grid = (-2:4/g:2);
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);
A = AR(idx2,S,pi);
eta = sigma*(1+Z.^2).*A.*trnd(3,n,1)/3 + (1+Z.^2).*(1-A).*trnd(3,n,1)/3;
Y = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;

elseif idx1 == 4
Z = randn(n,1)*2;
% m0 = -(gamma*Z.^2/4);
% m1= gamma*Z/2+ gamma*Z.^2/4;
m0 = -gamma*(Z/2).^2;
% m1 = 2*gamma*cos(Z);
m1 = gamma*(Z/2).^2;
% grid = (-2:4/g:2);
grid = 2*norminv((0:1/g:1));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);
A = AR(idx2,S,pi);
eta = sigma*(1+0.5*exp(-Z.^2/2)).*A.*randn(n,1) + (1+0.5*exp(-Z.^2/2)).*(1-A).*randn(n,1);
% eta = randn(n,1)*0.1;
Y = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;
end
