% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Inference for SBR assignment rule using DGP 1]

% idx1: index of the dgp. 
%       idx1=1 for method 1; idx1=3 for method 2.
% idx2: index of the assignment rule.
%       idx2=1 for SRS; idx2=2 for WEI; idx2=3 for BCD; idx2=4 for SBR.
% n: sample size
% g: number of intervals. 
% pi: pi, targeted assignment probability. 
% gamma: coefficient of Z. 
% sigma: standard error of the error term for Y(1)

clear
clc

tic
load('par_true')
R = 1000;B = 1000;
n = 200;g=4;mu0=0;mu1=0;pi=0.7;gamma=4;sigma = 2; Hlist = [0.5:0.1:1.5];H = length(Hlist);
taulist = [0.25,0.5,0.75];
T = zeros(4,4,R); thetahat = T; ind=T; T_t_boot = T;T_sfe_adj = T;T_adj = T;T_sfe_boot = T;beta = zeros(length(taulist),4,4,R);beta_sfe = beta;T_q_b = zeros(4,4,length(taulist),R);
T_q_b_sfe = T_q_b; std_T_naive = T;std_sfe_adj = T;std_T_adj = T;qhat = beta;

std_t_b = T;std_t_sfe = T; std_t_b_naive = T; std_t_sfe_naive = T;str_q_b_naive = T_q_b;std_t_b_ipw = T;std_t_ipw_naive = T;

str_q_b = T_q_b;str_q_b_sfe = T_q_b; std_q_b_naive = T_q_b;str_q_b_sfe_naive = T_q_b;str_q_b_ipw = T_q_b;str_q_b_ipw_naive = T_q_b;T_q_b_ipw_naive = T_q_b;

std_q_naive = T_q_b;T_q_naive = T_q_b;

cv_t_b_naive = zeros(4,4,R,2); cv_t_sfe_naive = cv_t_b_naive; cv_t_ipw_naive = cv_t_b_naive;
cv_q_b_naive = zeros(4,4,length(taulist),R,2); cv_q_sfe_naive = cv_q_b_naive; cv_q_ipw_naive = cv_q_b_naive;


T_t_boot_naive = T;T_sfe_boot_naive = T; T_ipw_boot_naive = T;T_q_b_naive =T_q_b;T_q_b_sfe_naive = T_q_b;
T_q_b_ipw = T_q_b_sfe; T_t_b_ipw = T;
std_q_adj  = zeros(4,4,length(taulist),R,H);T_q_adj = std_q_adj;

theta_sfe = T; thetahat_ipw = T;
tauvar = [pi*(1-pi),1/12,0,0];
ker = @(u) normpdf(u);
for r = 1:R
rng(r);
for idx1 = 1
    for idx2 = 2
[Y,S,A] = func_DGP(idx1,idx2,n,g,mu0,mu1,pi,gamma,sigma);
%% Naive t-stat
n1 = sum(A);n0 = n - n1;
Y1 = Y(A==1);Y0 = Y(A==0);
Y1bar = sum(Y.*A)/n1;Y0bar = sum(Y.*(1-A))/n0;
sigma21 = sum((Y - Y1bar).^2.*A)/n1;sigma20 = sum((Y - Y0bar).^2.*(1-A))/n0;
thetahat(idx1,idx2,r) = Y1bar - Y0bar;

std_T_naive(idx1,idx2,r) = sqrt(sigma21/n1 + sigma20/n0);
T(idx1,idx2,r) = (Y1bar - Y0bar - theta_true(idx1,idx2))/sqrt(sigma21/n1 + sigma20/n0);
for t = 1:length(taulist)
    tau = taulist(t);
    beta(t,idx1,idx2,r) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
end

%% SFE adj
FE = zeros(n,g);mus1 = zeros(g,1);mus0 = zeros(g,1);n1s = zeros(g,1);n0s = zeros(g,1);
for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum(Y.*A.*(S==s))/n1s(s);mus0(s) = sum(Y.*(1-A).*(S==s))/n0s(s);
end
X = [A,FE];
b = regress(Y,X);
theta_sfe(idx1,idx2,r) = b(1);
xiYbar = (sum(Y.^2.*A)/n1 - sum( (n1s+n0s)/n.*mus1.^2))/pi + (sum(Y.^2.*(1-A))/n0 - sum( (n1s+n0s)/n.*mus0.^2))/(1-pi);
xiH = sum((n1s+n0s)/n.*((mus1-Y1bar) - (mus0-Y0bar)).^2 );
xipi = (1-2*pi)^2/(pi^2*(1-pi)^2)*sum(tauvar(idx2)*(n1s+n0s)/n.*((mu1-Y1bar) - (mu0-Y0bar)).^2 );
std_sfe_adj(idx1,idx2,r) = sqrt((xiYbar + xiH+xipi)/n);
T_sfe_adj(idx1,idx2,r) = (theta_sfe(idx1,idx2,r) - theta_true(idx1,idx2))/sqrt((xiYbar + xiH+xipi)/n);

Atilde = zeros(n,1);


for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end

P = A-Atilde;

thetahat_ipw(idx1,idx2,r) = mean(A.*Y./P)-mean((1-A).*Y./(1-P));

for t= 1:length(taulist)
    tau = taulist(t);
betatemp = rq_fnm([ones(n,1),Atilde], Y, tau);
beta_sfe(t,idx1,idx2,r) = betatemp(2);

qhat(t,idx1,idx2,r) = myqr1(Y,A,P,tau*n,ones(n,1))-myqr0(Y,A,P,tau*n,ones(n,1));

end

%% T-stat adj

xiA = sum( tauvar(idx2)*(n1s+n0s)/n.*( (mus1-Y1bar)/pi + (mus0-Y0bar )/(1-pi)).^2);
std_T_adj(idx1,idx2,r) = sqrt((xiYbar + xiH+xiA)/n);
T_adj(idx1,idx2,r) = (Y1bar - Y0bar - theta_true(idx1,idx2))/sqrt((xiYbar + xiH+xiA)/n);

%% Quantile naive
for t = 1:length(taulist)
    tau = taulist(t);
FE = zeros(n,g);mus1 = zeros(g,1);mus0 = zeros(g,1);n1s = zeros(g,1);n0s = zeros(g,1);
q1= prctile(Y1,100*tau);q0 = prctile(Y0,100*tau);h1 = 1.06*n1^(-1/5)*std(Y1);h0 = 1.06*n0^(-1/5)*std(Y0); 
for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

f1 = mean(ker((Y1-q1)/h1)/h1);f0 = mean(ker((Y0-q0)/h0)/h0);

xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 +  (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*pi*(1-pi).*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
xipi = sum((n1s+n0s)/n.*(mus1/f1-mus0/f0).^2);
std_q_naive(idx1,idx2,t,r) = sqrt(xiYbar + xiH+xipi)/sqrt(n); 
T_q_naive(idx1,idx2,t,r) = (squeeze(beta(t,idx1,idx2,r)) - squeeze(beta_true(t,idx1,idx2)))./(sqrt(xiYbar + xiH+xipi)/sqrt(n));
end







%% Quantile adj
for t = 1:length(taulist)
    tau = taulist(t);
FE = zeros(n,g);mus1 = zeros(g,1);mus0 = zeros(g,1);n1s = zeros(g,1);n0s = zeros(g,1);
q1= prctile(Y1,100*tau);q0 = prctile(Y0,100*tau);h1 = 1.06*n1^(-1/5)*std(Y1);h0 = 1.06*n0^(-1/5)*std(Y0); 
for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

for h = 1:H;
h11 = Hlist(h)*h1;h00 = Hlist(h)*h0;    
f1 = mean(ker((Y1-q1)/h11)/h11);f0 = mean(ker((Y0-q0)/h00)/h00);

xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 +  (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*tauvar(idx2).*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
xipi = sum((n1s+n0s)/n.*(mus1/f1-mus0/f0).^2);
std_q_adj(idx1,idx2,t,r,h) = sqrt(xiYbar + xiH+xipi)/sqrt(n); 
T_q_adj(idx1,idx2,t,r,h) = (squeeze(beta(t,idx1,idx2,r)) - squeeze(beta_true(t,idx1,idx2)))./(sqrt(xiYbar + xiH+xipi)/sqrt(n));
end
end






%% Naive Bootstrap
thetahatstar = zeros(B,1);thetaSFEstar = thetahatstar; thetahatstar_ipw_naive=thetahatstar;betastar = zeros(B,length(taulist));betastar_sfe = zeros(B,length(taulist));
qhatstar_naive = betastar_sfe;
for b = 1:B
FEstar = zeros(n,g);
xi = -log(1-rand(n,1));


q1 = zeros(n,g);q0 = zeros(n,g);
for s = 1:g
       q1(:,s) = (S==s).*A;q0(:,s) = (S==s).*(1-A);
end
n1star = sum(A.*xi);n0star = sum((1-A).*xi);n1stars = zeros(g,1);nstars = n1stars;
for s = 1:g
    n1stars(s) = sum(xi.*(S==s).*A);
    nstars(s) = sum(xi.*(S==s)); 
end

Atildestar = zeros(n,1);
for i = 1:n
    Atildestar(i) = A(i) - n1stars(S(i))/nstars(S(i));
end

Y1barstar = sum(Y.*A.*xi)/n1star;Y0barstar = sum(Y.*(1-A).*xi)/n0star;

thetahatstar(b) = Y1barstar - Y0barstar;


thetaSFEstar(b) = sum(xi.*Y.*Atildestar)/sum(xi.*Atildestar.^2);
betastar_temp = zeros(length(taulist),1);betastar_sfe_temp = betastar_temp;qhatstar_temp = betastar_temp;
Pstar = A - Atildestar; 

thetahatstar_ipw_naive(b) = sum(xi.*A.*Y./Pstar)/sum(xi)-sum((1-A).*xi.*Y./(1-Pstar))/sum(xi);
for t = 1:length(taulist)
    tau = taulist(t);
betatemp = rq_fnm([ones(n,1).*xi,A.*xi], Y.*xi, tau);    
betastar_temp(t) = betatemp(2);
betatemp = rq_fnm([ones(n,1).*xi,Atildestar.*xi], Y.*xi, tau);
betastar_sfe_temp(t) = betatemp(2);

qhatstar_temp(t) = myqr1(Y,A,Pstar,tau*n,xi)-myqr0(Y,A,Pstar,tau*n,xi);
end
qhatstar_naive(b,:) = qhatstar_temp;
betastar(b,:) = betastar_temp;
betastar_sfe(b,:) = betastar_sfe_temp;
end
std_t_b_naive(idx1,idx2,r) = (prctile(thetahatstar,97.5) - prctile(thetahatstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_sfe_naive(idx1,idx2,r) = (prctile(thetaSFEstar,97.5) - prctile(thetaSFEstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_ipw_naive(idx1,idx2,r) = (prctile(thetahatstar_ipw_naive,97.5) - prctile(thetahatstar_ipw_naive,2.5))/(norminv(0.975) - norminv(0.025));

str_q_b_naive(idx1,idx2,:,r) =  (prctile(betastar,97.5,1) - prctile(betastar,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_sfe_naive(idx1,idx2,:,r) =  (prctile(betastar_sfe,97.5,1) - prctile(betastar_sfe,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_ipw_naive(idx1,idx2,:,r) =  (prctile(qhatstar_naive,97.5,1) - prctile(qhatstar_naive,2.5,1))/(norminv(0.975) - norminv(0.025));



T_t_boot_naive(idx1,idx2,r) = (Y1bar - Y0bar - theta_true(idx1,idx2))/std_t_b_naive(idx1,idx2,r);
T_sfe_boot_naive(idx1,idx2,r) = (theta_sfe(idx1,idx2,r) - theta_true(idx1,idx2))/std_t_sfe_naive(idx1,idx2,r);
T_ipw_boot_naive(idx1,idx2,r) = (squeeze(thetahat_ipw(idx1,idx2,r))-theta_true(idx1,idx2))/std_t_ipw_naive(idx1,idx2,r);



T_q_b_naive(idx1,idx2,:,r) = (squeeze(beta(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b_naive(idx1,idx2,:,r));
T_q_b_sfe_naive(idx1,idx2,:,r) = (squeeze(beta_sfe(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b_sfe_naive(idx1,idx2,:,r));
T_q_b_ipw_naive(idx1,idx2,:,r) = (squeeze(qhat(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b_ipw_naive(idx1,idx2,:,r));




%% Covariate-adaptive Bootstrap 
thetahatstar = zeros(B,1);thetaSFEstar = thetahatstar; betastar = zeros(B,length(taulist));betastar_sfe = zeros(B,length(taulist));
thetahatstar_ipw = thetahatstar;qhatstar = betastar_sfe;
for b = 1:B
FEstar = zeros(n,g);
Sstar = S(randsample(n,n,true),:);
Astar = AR(idx2,Sstar,pi);
q1 = zeros(n,g);q0 = zeros(n,g);
for s = 1:g
       q1(:,s) = (S==s).*A;q0(:,s) = (S==s).*(1-A);
end
Ystar = zeros(n,1);   
n1star = sum(Astar);n0star = n - n1star;n1stars = zeros(g,1);nstars = n1stars;
for s = 1:g
    FEstar(:,s) = (Sstar==s);
    n1stars(s) = sum((Sstar==s).*Astar);
    nstars(s) = sum(Sstar==s); 
end

Atildestar = zeros(n,1);
for i = 1:n
    Atildestar(i) = Astar(i) - n1stars(Sstar(i))/nstars(Sstar(i));
    if Astar(i) == 1
       Ysample = Y(q1(:,Sstar(i))==1,:);
       Ystar(i) = datasample(Ysample,1);
    else
        Ysample = Y(q0(:,Sstar(i))==1,:);
        Ystar(i) = datasample(Ysample,1);
    end 
end

Y1barstar = sum(Ystar.*Astar)/n1star;Y0barstar = sum(Ystar.*(1-Astar))/n0star;
Y1star = Ystar(Astar==1);Y0star = Ystar(Astar==0);

thetahatstar(b) = Y1barstar - Y0barstar;

Xstar = [Astar,FEstar];
btemp = regress(Ystar,Xstar);
thetaSFEstar(b) = btemp(1);
betastar_temp = zeros(length(taulist),1);betastar_sfe_temp = betastar_temp;qhatstar_temp = betastar_temp;
Pstar = Astar - Atildestar; 

thetahatstar_ipw(b) = mean(Astar.*Ystar./Pstar)-mean((1-Astar).*Ystar./(1-Pstar));


for t = 1:length(taulist)
    tau = taulist(t);
betastar_temp(t) = prctile(Y1star,tau*100) - prctile(Y0star,tau*100);
betatemp = rq_fnm([ones(n,1),Atildestar], Ystar, tau);
betastar_sfe_temp(t) = betatemp(2);
qhatstar_temp(t) = myqr1(Ystar,Astar,Pstar,tau*n,ones(n,1))-myqr0(Ystar,Astar,Pstar,tau*n,ones(n,1));
end

betastar(b,:) = betastar_temp;
betastar_sfe(b,:) = betastar_sfe_temp;
qhatstar(b,:) = qhatstar_temp;
end
std_t_b(idx1,idx2,r) = (prctile(thetahatstar,97.5) - prctile(thetahatstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_b_ipw(idx1,idx2,r) = (prctile(thetahatstar_ipw,97.5) - prctile(thetahatstar_ipw,2.5))/(norminv(0.975) - norminv(0.025));
std_t_sfe(idx1,idx2,r) = (prctile(thetaSFEstar,97.5) - prctile(thetaSFEstar,2.5))/(norminv(0.975) - norminv(0.025));
str_q_b(idx1,idx2,:,r) =  (prctile(betastar,97.5,1) - prctile(betastar,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_sfe(idx1,idx2,:,r) =  (prctile(betastar_sfe,97.5,1) - prctile(betastar_sfe,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_ipw(idx1,idx2,:,r) =  (prctile(qhatstar,97.5,1) - prctile(qhatstar,2.5,1))/(norminv(0.975) - norminv(0.025));

T_t_b_ipw(idx1,idx2,r) = (squeeze(thetahat_ipw(idx1,idx2,r))-theta_true(idx1,idx2))/std_t_b_ipw(idx1,idx2,r);
T_t_boot(idx1,idx2,r) = (Y1bar - Y0bar - theta_true(idx1,idx2))/std_t_b(idx1,idx2,r);
T_sfe_boot(idx1,idx2,r) = (theta_sfe(idx1,idx2,r) - theta_true(idx1,idx2))/std_t_sfe(idx1,idx2,r);
T_q_b(idx1,idx2,:,r) = (squeeze(beta(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b(idx1,idx2,:,r));
T_q_b_sfe(idx1,idx2,:,r) = (squeeze(beta_sfe(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b_sfe(idx1,idx2,:,r));
T_q_b_ipw(idx1,idx2,:,r) = (squeeze(qhat(:,idx1,idx2,r)) - squeeze(beta_true(:,idx1,idx2)))./squeeze(str_q_b_ipw(idx1,idx2,:,r));



    end
end

end
toc

save('H200_1_2','T_q_naive','T_q_adj','T_q_b_naive','T_q_b_ipw_naive','T_q_b','T_q_b_ipw','std_q_naive','std_q_adj','str_q_b_naive',...
    'str_q_b_ipw_naive','str_q_b','str_q_b_ipw','taulist','H','Hlist');