% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Generate True Data]

% idx1: index of the dgp. 
%       idx1=1 for method 1; idx1=3 for method 2.
% idx2: index of the assignment rule.
%       idx2=1 for SRS; idx2=2 for WEI; idx2=3 for BCD; idx2=4 for SBR.
% n: sample size
% g: number of intervals. 
% pi: pi, targeted assignment probability. 
% gamma: coefficient of Z. 
% sigma: standard error of the error term for Y(1)


clear
clc

n = 100000;g=4;mu0=0;mu1=0;pi=0.7;gamma=4;sigma = 2;R = 1000;
taulist = [0.25,0.5,0.75];
theta = zeros(4,R);beta = zeros(length(taulist),4,R);
tic
for r=1:R
 beta_temp = zeros(length(taulist),4);theta_temp = zeros(4,1);   
for idx1 = 1:4
A = ones(n,1);   
if idx1 == 1
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y1 = mu0+(mu1-mu0)*A+gamma*Z + eta;

elseif idx1 == 2
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y1 = mu0+(mu1-mu0)*A+gamma*Z.*A - gamma*(1-A).*(log(Z+3).*(Z<=0.5)) + eta;

elseif idx1 == 3
Z = 4*(rand(n,1)-0.5);
m0 = gamma*Z.^2.*(Z<=1).*(Z>=-1)+ gamma*(2-Z.^2).*(1-(Z<=1).*(Z>=-1))/4;
m1 = -m0;
grid = (-2:4/g:2);
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*(1+Z.^2).*A.*trnd(3,n,1)/3 + (1+Z.^2).*(1-A).*trnd(3,n,1)/3;
Y1 = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;

elseif idx1 == 4
% Z = 4*(rand(n,1)-0.5);
% m0 = gamma*Z.^2.*(Z<=1).*(Z>=-1)/2+ gamma*Z.*(1-(Z<=1).*(Z>=-1))/2;
% m0=1+gamma*Z+ gamma*Z.^2/2;
Z = randn(n,1)*2;
% m0 = -(gamma*Z.^2/4);
% m1= gamma*Z/2+ gamma*Z.^2/4;
m0 = -gamma*(Z/2).^2;
% m1 = 2*gamma*cos(Z);
m1 = gamma*(Z/2).^2;

% grid = (-2:4/g:2);
grid = 2*norminv((0:1/g:1));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*(1+0.5*exp(-Z.^2/2)).*A.*randn(n,1) + (1+0.5*exp(-Z.^2/2)).*(1-A).*randn(n,1);
% eta = randn(n,1)*0.1;
% eta = sigma*(1+Z.^2).*A.*trnd(3,n,1)/3 + (1+Z.^2).*(1-A).*trnd(3,n,1)/3;
Y1 = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;

end

A = zeros(n,1);   
if idx1 == 1
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y0 = mu0+(mu1-mu0)*A+gamma*Z + eta;

elseif idx1 == 2
Z = (random('beta',2,2,n,1)-0.5)*sqrt(20);
grid = (-0.5*sqrt(20):sqrt(20)/g:0.5*sqrt(20));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*A.*randn(n,1) + (1-A).*randn(n,1);
Y0 = mu0+(mu1-mu0)*A+gamma*Z.*A - gamma*(1-A).*(log(Z+3).*(Z<=0.5)) + eta;

elseif idx1 == 3
Z = 4*(rand(n,1)-0.5);
m0 = gamma*Z.^2.*(Z<=1).*(Z>=-1)+ gamma*(2-Z.^2).*(1-(Z<=1).*(Z>=-1))/4;
m1 = -m0;
grid = (-2:4/g:2);
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*(1+Z.^2).*A.*trnd(3,n,1)/3 + (1+Z.^2).*(1-A).*trnd(3,n,1)/3;
Y0 = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;

elseif idx1 == 4
Z = randn(n,1)*2;
% m0 = -(gamma*Z.^2/4);
% m1= gamma*Z/2+ gamma*Z.^2/4;
m0 = -gamma*(Z/2).^2;
% m1 = 2*gamma*cos(Z);
m1 = gamma*(Z/2).^2;

% grid = (-2:4/g:2);
grid = 2*norminv((0:1/g:1));
S = sum(repmat(Z,1,g)<repmat(grid(2:end),n,1),2);

eta = sigma*(1+0.5*exp(-Z.^2/2)).*A.*randn(n,1) + (1+0.5*exp(-Z.^2/2)).*(1-A).*randn(n,1);
% eta = randn(n,1)*0.1;
% eta = sigma*(1+Z.^2).*A.*trnd(3,n,1)/3 + (1+Z.^2).*(1-A).*trnd(3,n,1)/3;

Y0 = mu0+(mu1-mu0)*A+ A.*m1 + (1-A).*m0 + eta;

end

toc

%% 

for t = 1:length(taulist)
    tau = taulist(t);
    beta_temp(t,idx1) = prctile(Y1,100*tau) - prctile(Y0,100*tau); 
end
theta_temp(idx1) = mean(Y1) - mean(Y0);
 
end
beta(:,:,r) = beta_temp;
theta(:,r) = theta_temp;
end
toc
theta_true = repmat(mean(theta,2),1,4);beta_true = repmat(mean(beta,3),[1,1,4]);
%%
save('par_true','theta_true','beta_true')

